@@ -17,7 +17,7 @@
 """handles command line and config file option parsing for the gbp commands"""
 
 from optparse import OptionParser, OptionGroup, Option, OptionValueError
-from ConfigParser import SafeConfigParser, NoSectionError
+from configparser import SafeConfigParser, NoSectionError
 from copy import copy
 import os.path
 import tempfile
@@ -371,15 +371,15 @@ class GbpOptionParser(OptionParser):
 
         # Read per-tree config file
         if repo and git_treeish and filename.startswith('%(top_dir)s/'):
-            with tempfile.TemporaryFile() as tmp:
+            with tempfile.NamedTemporaryFile() as tmp:
                 relpath = filename.replace('%(top_dir)s/', '')
                 try:
                     config = repo.show('%s:%s' % (git_treeish, relpath))
-                    tmp.writelines(config)
+                    tmp.write(config)
                 except GitRepositoryError:
                     pass
                 tmp.seek(0)
-                parser.readfp(tmp)
+                parser.read(tmp.name)
                 return
         try:
             filename = filename % str_fields
@@ -427,11 +427,11 @@ class GbpOptionParser(OptionParser):
         if parser.has_section(cmd):
             # Don't use items() until we got rid of the compat sections
             # since this pulls in the defaults again
-            self.config.update(dict(parser._sections[cmd].items()))
+            self.config.update(dict(list(parser._sections[cmd].items())))
 
         for section in self.sections:
             if parser.has_section(section):
-                self.config.update(dict(parser._sections[section].items()))
+                self.config.update(dict(list(parser._sections[section].items())))
             else:
                 raise NoSectionError("Mandatory section [%s] does not exist."
                                      % section)
