@@ -19,14 +19,17 @@
 import subprocess
 import os.path
 import re
+import sys
 from collections import defaultdict
 import select
 
 import gbp.log as log
+from gbp.format import format_b
 from gbp.git.modifier import GitModifier
 from gbp.git.commit import GitCommit
 from gbp.git.errors import GitError
 from gbp.git.args import GitArgs
+from gbp.paths import to_bin
 
 
 class GitRepositoryError(GitError):
@@ -39,7 +42,7 @@ class GitRemote(object):
     def __init__(self, name, fetch_url, push_urls):
         self._name = name
         self._fetch_url = fetch_url
-        if isinstance(push_urls, basestring):
+        if isinstance(push_urls, str):
             self._push_urls = [push_urls]
         else:
             self._push_urls = [url for url in push_urls]
@@ -85,7 +88,7 @@ class GitRepository(object):
                 "Failed to get repository git dir at '%s'" % self.path)
 
         # Set git meta data dir
-        git_dir = out.strip()
+        git_dir = out.strip().decode(sys.getfilesystemencoding())
         if os.path.isabs(git_dir):
             self._git_dir = git_dir
         else:
@@ -98,7 +101,7 @@ class GitRepository(object):
         else:
             out, dummy, ret = self._git_inout('rev-parse', ['--show-toplevel'],
                                               capture_stderr=True)
-            self._path = os.path.abspath(out.strip())
+            self._path = os.path.abspath(out.decode().strip())
 
     def __init__(self, path):
         self._path = os.path.abspath(path)
@@ -106,9 +109,10 @@ class GitRepository(object):
             # Check for bare repository
             out, dummy, ret = self._git_inout('rev-parse', ['--is-bare-repository'],
                                               capture_stderr=True)
+            cdup = out.strip().decode(sys.getfilesystemencoding())
             if ret:
-                raise GitRepositoryError("No Git repository at '%s': '%s'" % (self.path, out))
-            self._bare = False if out.strip() != 'true' else True
+                raise GitRepositoryError("No Git repository at '%s': '%s'" % (self.path, out.decode().strip()))
+            self._bare = False if out.decode().strip() != 'true' else True
 
             self._check_dirs()
 
@@ -140,7 +144,7 @@ class GitRepository(object):
         @param cwd: directory to swith to when running the command, defaults to I{self.path}
         @type cwd: C{str}
         @return: stdout, return code
-        @rtype: C{tuple} of C{list} of C{str} and C{int}
+        @rtype: C{tuple} of C{list} of C{bytestr} and C{int}
 
         @deprecated: use L{gbp.git.repository.GitRepository._git_inout} instead.
         """
@@ -174,13 +178,13 @@ class GitRepository(object):
         @param capture_stderr: whether to capture stderr
         @type capture_stderr: C{bool}
         @return: stdout, stderr, return code
-        @rtype: C{tuple} of C{str}, C{str}, C{int}
+        @rtype: C{tuple} of C{bytestr}, C{bytestr}, C{int}
         """
         if not cwd:
             cwd = self.path
         ret = 0
-        stdout = ''
-        stderr = ''
+        stdout = b''
+        stderr = b''
         try:
             for outdata in self.__git_inout(command, args, input, extra_env,
                                             cwd, capture_stderr,
@@ -203,7 +207,7 @@ class GitRepository(object):
         """
         if not cwd:
             cwd = self.path
-        stderr = ''
+        stderr = b''
         try:
             for outdata in self.__git_inout(command, args, stdin, extra_env,
                                             cwd, capture_stderr, True):
@@ -262,8 +266,8 @@ class GitRepository(object):
                 if w_ind > len(stdin):
                     rm_polled_fd(popen.stdin, in_fds)
             # Read in chunks of 4k
-            stdout = popen.stdout.read(4096) if popen.stdout in ready[0] else ''
-            stderr = popen.stderr.read(4096) if popen.stderr in ready[0] else ''
+            stdout = popen.stdout.read(4096) if popen.stdout in ready[0] else b''
+            stderr = popen.stderr.read(4096) if popen.stderr in ready[0] else b''
             if popen.stdout in ready[0] and not stdout:
                 rm_polled_fd(popen.stdout, out_fds)
             if popen.stderr in ready[0] and not stderr:
@@ -298,9 +302,8 @@ class GitRepository(object):
         except Exception as excobj:
             raise GitRepositoryError("Error running git %s: %s" % (command, excobj))
         if ret:
-            raise GitRepositoryError("Error running git %s: %s" %
-                                        (command, stderr.strip()))
-
+            detail = stderr or stdout
+            raise GitRepositoryError("Error running git %s: %s" % (command, detail.decode().strip()))
 
     def _cmd_has_feature(self, command, feature):
         """
@@ -315,12 +318,12 @@ class GitRepository(object):
         """
         args = GitArgs(command, '-m')
         help, stderr, ret = self._git_inout('help',
-                                           args.args,
-                                           extra_env={'LC_ALL': 'C'},
-                                           capture_stderr=True)
+                                            args.args,
+                                            extra_env={'LC_ALL': 'C'},
+                                            capture_stderr=True)
         if ret:
             raise GitRepositoryError("Invalid git command '%s': %s"
-                                     % (command, stderr[:-1]))
+                                     % (command, stderr.decode().strip()))
 
         # Parse git command man page
         section_re = re.compile(r'^(?P<section>[A-Z].*)')
@@ -328,7 +331,7 @@ class GitRepository(object):
         optopt_re = re.compile(r'--\[(?P<prefix>[a-zA-Z\-]+)\]-?')
         backspace_re = re.compile(".\b")
         man_section = None
-        for line in help.splitlines():
+        for line in help.decode().splitlines():
             if man_section == "OPTIONS" and line.startswith('       -'):
                 opts = line.split(',')
                 for opt in opts:
@@ -359,7 +362,7 @@ class GitRepository(object):
 
     @property
     def bare(self):
-        """Wheter this is a bare repository"""
+        """Whether this is a bare repository"""
         return self._bare
 
     @property
@@ -377,7 +380,7 @@ class GitRepository(object):
 
     @property
     def head(self):
-        """return the SHA1 of the current HEAD"""
+        """SHA1 of the current HEAD"""
         return self.rev_parse('HEAD')
 
 #{ Branches and Merging
@@ -433,13 +436,13 @@ class GitRepository(object):
         @raises GitRepositoryError: if HEAD is not a symbolic ref
           (e.g. when in detached HEAD state)
         """
-        out, dummy, ret = self._git_inout('symbolic-ref', [ 'HEAD' ],
-                                           capture_stderr=True)
+        out, _, ret = self._git_inout('symbolic-ref', ['HEAD'],
+                                      capture_stderr=True)
         if ret:
             # We don't append stderr since
             # "fatal: ref HEAD is not a symbolic ref" confuses people
             raise GitRepositoryError("Currently not on a branch")
-        ref = out.split('\n')[0]
+        ref = out.decode().split('\n')[0]
 
         # Check if ref really exists
         try:
@@ -517,9 +520,9 @@ class GitRepository(object):
         args.add(commit2)
         sha1, stderr, ret = self._git_inout('merge-base', args.args, capture_stderr=True)
         if not ret:
-            return self.strip_sha1(sha1)
+            return self.strip_sha1(sha1.decode())
         else:
-            raise GitRepositoryError("Failed to get common ancestor: %s" % stderr.strip())
+            raise GitRepositoryError("Failed to get common ancestor: %s" % stderr.decode().strip())
 
     def merge(self, commit, verbose=False, edit=False):
         """
@@ -529,7 +532,7 @@ class GitRepository(object):
         @type commit: C{str}
         @param verbose: whether to print a summary after the merge
         @type verbose: C{bool}
-        @param edit: wheter to invoke an editor to edit the merge message
+        @param edit: whether to invoke an editor to edit the merge message
         @type edit: C{bool}
         """
         args = GitArgs()
@@ -559,14 +562,15 @@ class GitRepository(object):
         """
         has_local = False       # local repo has new commits
         has_remote = False      # remote repo has new commits
-        out = self._git_getoutput('rev-list', ["--left-right",
+        out = self._git_getoutput('rev-list',
+                                  ["--left-right",
                                    "%s...%s" % (from_branch, to_branch),
                                    "--"])[0]
 
-        if not out: # both branches have the same commits
+        if not out:  # both branches have the same commits
             return True, True
 
-        for line in out:
+        for line in (l.decode() for l in out):
             if line.startswith("<"):
                 has_local = True
             elif line.startswith(">"):
@@ -588,10 +592,10 @@ class GitRepository(object):
         @return: local or remote branches
         @rtype: C{list}
         """
-        args = [ '--format=%(refname:short)' ]
-        args += [ 'refs/remotes/' ] if remote else [ 'refs/heads/' ]
+        args = ['--format=%(refname:short)']
+        args += ['refs/remotes/'] if remote else ['refs/heads/']
         out = self._git_getoutput('for-each-ref', args)[0]
-        return [ ref.strip() for ref in out ]
+        return [ref.decode().strip() for ref in out]
 
     def get_local_branches(self):
         """
@@ -649,8 +653,8 @@ class GitRepository(object):
         args.add('--contains')
         args.add(commit)
 
-        out, ret =  self._git_getoutput('branch', args.args)
-        for line in out:
+        out, ret = self._git_getoutput('branch', args.args)
+        for line in [l.decode() for l in out]:
             # remove prefix '*' for current branch before comparing
             line = line.replace('*', '')
             if line.strip() == branch:
@@ -703,7 +707,7 @@ class GitRepository(object):
 
         out = self._git_getoutput('for-each-ref', args.args)[0]
 
-        return out[0].strip()
+        return out[0].decode().strip()
 
 #{ Tags
 
@@ -802,9 +806,9 @@ class GitRepository(object):
         tag, err, ret = self._git_inout('describe', args.args,
                                         capture_stderr=True)
         if ret:
-            raise GitRepositoryError("Can't describe %s. Git error: %s" % \
-                                         (commitish, err.strip()))
-        return tag.strip()
+            raise GitRepositoryError("Can't describe %s. Git error: %s" %
+                                     (commitish, err.decode().strip()))
+        return tag.decode().strip()
 
     def find_tag(self, commit, pattern=None):
         """
@@ -828,8 +832,8 @@ class GitRepository(object):
         @return: tags
         @rtype: C{list} of C{str}
         """
-        args = [ '-l', pattern ] if pattern else []
-        return [ line.strip() for line in self._git_getoutput('tag', args)[0] ]
+        args = ['-l', pattern] if pattern else []
+        return [line.decode().strip() for line in self._git_getoutput('tag', args)[0]]
 
     def verify_tag(self, tag):
         """
@@ -902,7 +906,7 @@ class GitRepository(object):
             # Get a more helpful error message.
             out = self._status(porcelain=False,
                                 ignore_untracked=ignore_untracked)
-            return (False, "".join(out))
+            return (False, "".join([e.decode() for e in out]))
         else:
             return (True, '')
 
@@ -946,17 +950,17 @@ class GitRepository(object):
         if ret:
             raise GitRepositoryError("Can't get repository status: %s" % err)
 
-        elements = out.split('\x00')
+        elements = out.split(b'\x00')
         result = defaultdict(list)
 
-        while elements[0] != '':
+        while elements[0] != b'':
             element = elements.pop(0)
-            status = element[:2]
+            status = element[:2].decode()
             filepath = element[3:]
             # Expect to have two filenames for renames and copies
             if status[0] in ['R', 'C']:
-                filepath = elements.pop(0) + '\x00' + filepath
-            result[status].append(filepath)
+                filepath = elements.pop(0) + b'\x00' + filepath
+            result[status].append(filepath.decode())
 
         return result
 
@@ -989,7 +993,7 @@ class GitRepository(object):
                                             capture_stderr=True)
         if ret:
             raise GitRepositoryError("revision '%s' not found" % name)
-        return self.strip_sha1(sha.splitlines()[0], short)
+        return self.strip_sha1(sha.decode().splitlines()[0], short)
 
     @staticmethod
     def strip_sha1(sha1, length=0):
@@ -1002,7 +1006,7 @@ class GitRepository(object):
         >>> GitRepository.strip_sha1('58ef37d', 10)
         Traceback (most recent call last):
         ...
-        GitRepositoryError: '58ef37d' is not a valid sha1 of length 10
+        gbp.git.repository.GitRepositoryError: '58ef37d' is not a valid sha1 of length 10
         >>> GitRepository.strip_sha1('58ef37d', 7)
         '58ef37d'
         >>> GitRepository.strip_sha1('123456789', 7)
@@ -1010,16 +1014,16 @@ class GitRepository(object):
         >>> GitRepository.strip_sha1('foobar')
         Traceback (most recent call last):
         ...
-        GitRepositoryError: 'foobar' is not a valid sha1
+        gbp.git.repository.GitRepositoryError: 'foobar' is not a valid sha1
         """
         maxlen = 40
         s = sha1.strip()
 
-        l = length or maxlen
+        sl = length or maxlen
 
-        if len(s) < l or len(s) > maxlen:
+        if len(s) < sl or len(s) > maxlen:
             raise GitRepositoryError("'%s' is not a valid sha1%s" %
-                                     (s, " of length %d" % l if length else ""))
+                                     (s, " of length %d" % sl if length else ""))
         return s
 
 #{ Trees
@@ -1041,9 +1045,9 @@ class GitRepository(object):
         @return: C{True} if the repository has that tree, C{False} otherwise
         @rtype: C{bool}
         """
-        _out, _err, ret =  self._git_inout('ls-tree', [treeish],
-                                           capture_stderr=True)
-        return [ True, False ][ret != 0]
+        _out, _err, ret = self._git_inout('ls-tree', [treeish],
+                                          capture_stderr=True)
+        return [True, False][ret != 0]
 
     def write_tree(self, index_file=None):
         """
@@ -1063,28 +1067,30 @@ class GitRepository(object):
                                             extra_env=extra_env,
                                             capture_stderr=True)
         if ret:
-            raise GitRepositoryError("Can't write out current index: %s" % stderr[:-1])
-        return tree.strip()
+            raise GitRepositoryError("Can't write out current index: %s" % stderr.decode().strip())
+        return tree.decode().strip()
 
     def make_tree(self, contents):
         """
-        Create a tree based on contents. I{contents} has the same format than
-        the I{GitRepository.list_tree} output.
+        Create a tree based on contents.
+
+        @param contents: same format as I{GitRepository.list_tree} output.
+        @type contents: C{list} of C{str}
         """
-        out=''
+        objs = b''
         args = GitArgs('-z')
 
-        for obj in contents:
-             mode, type, sha1, name = obj
-             out += '%s %s %s\t%s\0' % (mode, type, sha1, name)
+        for mode, type_, sha1, name in contents:
+            name = to_bin(name)
+            objs += format_b(b'%s %s %s\t%s\0', mode.encode(), type_.encode(), sha1.encode(), name)
 
-        sha1, err, ret =  self._git_inout('mktree',
-                                          args.args,
-                                          out,
-                                          capture_stderr=True)
+        sha1, err, ret = self._git_inout('mktree',
+                                         args.args,
+                                         objs,
+                                         capture_stderr=True)
         if ret:
             raise GitRepositoryError("Failed to mktree: '%s'" % err)
-        return self.strip_sha1(sha1)
+        return self.strip_sha1(sha1.decode())
 
     def get_obj_type(self, obj):
         """
@@ -1098,12 +1104,12 @@ class GitRepository(object):
         out, ret = self._git_getoutput('cat-file', args=['-t', obj])
         if ret:
             raise GitRepositoryError("Not a Git repository object: '%s'" % obj)
-        return out[0].strip()
+        return out[0].decode().strip()
 
     def list_tree(self, treeish, recurse=False, paths=None):
         """
         Get a trees content. It returns a list of objects that match the
-        'ls-tree' output: [ mode, type, sha1, path ].
+        'ls-tree' output: [mode, type, sha1, path].
 
         @param treeish: the treeish object to list
         @type treeish: C{str}
@@ -1118,14 +1124,18 @@ class GitRepository(object):
         args.add("--")
         args.add_cond(paths, paths)
 
-        out, err, ret =  self._git_inout('ls-tree', args.args, capture_stderr=True)
+        out, err, ret = self._git_inout('ls-tree', args.args, capture_stderr=True)
         if ret:
-            raise GitRepositoryError("Failed to ls-tree '%s': '%s'" % (treeish, err))
+            raise GitRepositoryError("Failed to ls-tree '%s': '%s'" % (treeish, err.decode().strip()))
 
         tree = []
-        for line in out.split('\0'):
+        for line in out.split(b'\0'):
             if line:
-                tree.append(line.split(None, 3))
+                parts = line.split(None, 3)
+                # decode everything but the file name
+                for i in range(len(parts) - 1):
+                    parts[i] = parts[i].decode()
+                tree.append(parts)
         return tree
 
 #}
@@ -1139,8 +1149,9 @@ class GitRepository(object):
         @rtype: C{str}
         """
         value, ret = self._git_getoutput('config', [ name ])
-        if ret: raise KeyError
-        return value[0][:-1] # first line with \n ending removed
+        if ret:
+            raise KeyError("'%s' not found in git config")
+        return value[0].decode()[:-1]  # first line with \n ending removed
 
     def get_author_info(self):
         """
@@ -1151,11 +1162,11 @@ class GitRepository(object):
         @rtype: L{GitModifier}
         """
         try:
-           name =  self.get_config("user.name")
+            name =  self.get_config("user.name")
         except KeyError:
-           name = os.getenv("USER")
+            name = os.getenv("USER")
         try:
-           email =  self.get_config("user.email")
+            email = self.get_config("user.email")
         except KeyError:
             email = os.getenv("EMAIL")
         email = os.getenv("GIT_AUTHOR_EMAIL", email)
@@ -1173,19 +1184,19 @@ class GitRepository(object):
         """
         out, err, ret = self._git_inout('remote', [], capture_stderr=True)
         if ret:
-            raise GitRepositoryError('Failed to get list of remotes: %s' % err)
+            raise GitRepositoryError('Failed to get list of remotes: %s' % err.decode().strip())
 
         # Get information about all remotes
         remotes = {}
-        for remote in out.splitlines():
+        for remote in out.decode().splitlines():
             out, err, _ret = self._git_inout('remote', ['show', '-n', remote],
                                              capture_stderr=True)
             if ret:
                 raise GitRepositoryError('Failed to get information for remote '
-                                         '%s: %s' % (remote, err))
+                                         '%s: %s' % (remote, err.decode().strip()))
             fetch_url = None
             push_urls = []
-            for line in out.splitlines():
+            for line in out.decode().splitlines():
                 match = re.match('\s*Fetch\s+URL:\s*(\S.*)', line)
                 if match:
                     fetch_url = match.group(1)
@@ -1208,11 +1219,11 @@ class GitRepository(object):
         stdout, stderr, ret = self._git_inout('remote', ['-v'],
                                               capture_stderr=True)
         if ret:
-            raise GitRepositoryError('Failed to get remotes: %s' % stderr)
+            raise GitRepositoryError('Failed to get remotes: %s' % stderr.decode().strip())
 
         remotes = {}
         for rem in stdout.splitlines():
-            name, url_urltype = rem.split('\t', 1)
+            name, url_urltype = rem.decode().strip().split('\t', 1)
             url, urltype = url_urltype.rsplit(' ', 1)
             urltype = urltype.strip('()')
             if not name in remotes:
@@ -1392,7 +1403,7 @@ class GitRepository(object):
         @param verbose: be verbose
         @type verbose: C{bool}
         """
-        if isinstance(paths, basestring):
+        if isinstance(paths, str):
             paths = [ paths ]
 
         args =  [] if verbose else ['--quiet']
@@ -1404,12 +1415,12 @@ class GitRepository(object):
 
         @param types: list of types to show
         @type types: C{list}
-        @return: list of files
+        @return: list of files as byte string
         @rtype: C{list} of C{str}
         """
-        all_types = [ 'cached', 'deleted', 'others', 'ignored',  'stage'
-                      'unmerged', 'killed', 'modified' ]
-        args = [ '-z' ]
+        all_types = ['cached', 'deleted', 'others', 'ignored', 'stage'
+                     'unmerged', 'killed', 'modified']
+        args = ['-z']
 
         for t in types:
             if t in all_types:
@@ -1420,7 +1431,7 @@ class GitRepository(object):
         if ret:
             raise GitRepositoryError("Error listing files: '%d'" % ret)
         if out:
-            return [ file for file in out[0].split('\0') if file ]
+            return [file for file in out[0].split(b'\0') if file]
         else:
             return []
 
@@ -1430,7 +1441,7 @@ class GitRepository(object):
         Hash a single file and write it into the object database
 
         @param filename: the filename to the content of the file to hash
-        @type filename: C{str}
+        @type filename: C{bytestr}
         @param filters: whether to run filters
         @type filters: C{bool}
         @return: the hash of the file
@@ -1444,10 +1455,9 @@ class GitRepository(object):
                                             args.args,
                                             capture_stderr=True)
         if not ret:
-            return self.strip_sha1(sha1)
+            return self.strip_sha1(sha1.decode())
         else:
-            raise GitRepositoryError("Failed to hash %s: %s" % (filename,
-                                                                stderr))
+            raise GitRepositoryError("Failed to hash %s: %s" % (filename, stderr.decode().strip()))
 #}
 
 #{ Comitting
@@ -1594,10 +1604,10 @@ class GitRepository(object):
         @type committer: C{dict} with keys 'name' and 'email'
         """
         extra_env = {}
-        for key, val in author.items():
+        for key, val in list(author.items()):
             if val:
                 extra_env['GIT_AUTHOR_%s' % key.upper()] = val
-        for key, val in committer.items():
+        for key, val in list(committer.items()):
             if val:
                 extra_env['GIT_COMMITTER_%s' % key.upper()] = val
 
@@ -1606,13 +1616,13 @@ class GitRepository(object):
             args += [ '-p' , parent ]
         sha1, stderr, ret = self._git_inout('commit-tree',
                                             args,
-                                            msg,
+                                            msg.encode(),
                                             extra_env,
                                             capture_stderr=True)
         if not ret:
-            return self.strip_sha1(sha1)
+            return self.strip_sha1(sha1.decode())
         else:
-            raise GitRepositoryError("Failed to commit tree: %s" % stderr)
+            raise GitRepositoryError("Failed to commit tree: %s" % stderr.decode().strip())
 
 #{ Commit Information
 
@@ -1644,23 +1654,27 @@ class GitRepository(object):
             args.add(until)
         args.add_cond(options, options)
         args.add("--")
-        if isinstance(paths, basestring):
-            paths = [ paths ]
+        if isinstance(paths, str):
+            paths = [paths]
         args.add_cond(paths, paths)
 
         commits, ret = self._git_getoutput('log', args.args)
         if ret:
             where = " on %s" % paths if paths else ""
             raise GitRepositoryError("Error getting commits %s..%s%s" %
-                        (since, until, where))
-        return [ commit.strip() for commit in commits ]
+                                     (since, until, where))
+        return [commit.decode().strip() for commit in commits]
 
     def show(self, id):
-        """git-show id"""
+        """
+        Show a git object
+
+        @rtype: C{bytestr}
+        """
         obj, stderr, ret = self._git_inout('show', ["--pretty=medium", id],
-                                              capture_stderr=True)
+                                           capture_stderr=True)
         if ret:
-            raise GitRepositoryError("can't get %s: %s" % (id, stderr.rstrip()))
+            raise GitRepositoryError("can't get %s: %s" % (id, stderr.decode().rstrip()))
         return obj
 
     def grep_log(self, regex, since=None):
@@ -1682,9 +1696,9 @@ class GitRepository(object):
                                               capture_stderr=True)
         if ret:
             raise GitRepositoryError("Error grepping log for %s: %s" %
-                                     (regex, stderr[:-1]))
+                                     (regex, stderr.decode().strip()))
         if stdout:
-            return [ commit.strip() for commit in stdout.split('\n')[::-1] ]
+            return [commit.strip() for commit in stdout.decode().split('\n')[::-1]]
         else:
             return []
 
@@ -1713,36 +1727,38 @@ class GitRepository(object):
         args = GitArgs('--pretty=format:%an%x00%ae%x00%ad%x00%cn%x00%ce%x00%cd%x00%s%x00%f%x00%b%x00',
                        '-z', '--date=raw', '--no-renames', '--name-status',
                        commit_sha1)
-        out, err, ret =  self._git_inout('show', args.args)
+        out, err, ret = self._git_inout('show', args.args)
         if ret:
             raise GitRepositoryError("Unable to retrieve commit info for %s"
                                      % commitish)
 
-        fields = out.split('\x00')
+        fields = out.split(b'\x00')
 
-        author = GitModifier(fields[0].strip(),
-                             fields[1].strip(),
-                             fields[2].strip())
-        committer = GitModifier(fields[3].strip(),
-                                fields[4].strip(),
-                                fields[5].strip())
+        author = GitModifier(fields[0].decode().strip(),
+                             fields[1].decode().strip(),
+                             fields[2].decode().strip())
+        committer = GitModifier(fields[3].decode().strip(),
+                                fields[4].decode().strip(),
+                                fields[5].decode().strip())
 
         files = defaultdict(list)
         file_fields = fields[9:]
-        # For some reason git returns one extra empty field for merge commits
-        if file_fields[0] == '': file_fields.pop(0)
-        while len(file_fields) and file_fields[0] != '':
-            status = file_fields.pop(0).strip()
-            path = file_fields.pop(0)
-            files[status].append(path)
 
-        return {'id' : commitish,
-                'author' : author,
-                'committer' : committer,
-                'subject' : fields[6],
-                'patchname' : fields[7],
-                'body' : fields[8],
-                'files' : files}
+        # For some reason git returns one extra empty field for merge commits
+        if file_fields[0] == b'':
+            file_fields.pop(0)
+        while len(file_fields) and file_fields[0] != b'':
+            status = file_fields.pop(0).decode().strip()
+            path = file_fields.pop(0)
+            files[status].append(path.decode())
+
+        return {'id': commitish,
+                'author': author,
+                'committer': committer,
+                'subject': fields[6].decode(),
+                'patchname': fields[7].decode(),
+                'body': fields[8].decode(),
+                'files': files}
 
 #{ Patches
     def format_patches(self, start, end, output_dir,
@@ -1803,7 +1819,7 @@ class GitRepository(object):
         @param ignore_submodules: ignore changes to submodules
         @type ignore_submodules: C{bool}
         @return: diff
-        @rtype: C{str}
+        @rtype: C{binary}
         """
         options = GitArgs('-p', '--no-ext-diff')
         if stat is True:
@@ -1836,16 +1852,16 @@ class GitRepository(object):
         options = GitArgs('--name-status', '-z', obj1, obj2)
         output, stderr, ret = self._git_inout('diff', options.args)
 
-        elements = output.split('\x00')
+        elements = output.split(b'\x00')
         result = defaultdict(list)
 
-        while elements[0] != '':
-            status = elements.pop(0)[0]
-            filepath = elements.pop(0)
+        while elements[0] != b'':
+            status = elements.pop(0).decode()[0]
+            filepath = elements.pop(0).decode()
             # Expect to have two filenames for renames and copies
             if status in ['R', 'C']:
                 result[status].append(filepath)
-                filepath = elements.pop(0)
+                filepath = elements.pop(0).decode()
             result[status].append(filepath)
 
         return result
@@ -1879,8 +1895,7 @@ class GitRepository(object):
         if output:
             out, err, ret = self._git_inout('archive', args.args)
             if ret:
-                raise GitRepositoryError("Unable to archive %s: %s" % (treeish,
-                                                                       err))
+                raise GitRepositoryError("Unable to archive %s: %s" % (treeish, err.decode().strip()))
         else:
             return self._git_inout2('archive', args.args)
 
@@ -1969,13 +1984,16 @@ class GitRepository(object):
             args += ['-r']
 
         out, ret =  self._git_getoutput('ls-tree', args, cwd=path)
-        for line in out:
-            mode, objtype, commit, name = line[:-1].split(None, 3)
+        for line in out.split(b'\n'):
+            if not line:
+                continue
+            mode, objtype, commit, name = line.decode().split(None, 3)
+
             # A submodules is shown as "commit" object in ls-tree:
             if objtype == "commit":
                 nextpath = os.path.join(path, name)
-                submodules.append( (nextpath.replace(self.path,'').lstrip('/'),
-                                    commit) )
+                submodules.append((nextpath.replace(self.path, '').lstrip('/'),
+                                   commit))
                 if recursive:
                     submodules += self.get_submodules(commit, path=nextpath,
                                                       recursive=recursive)
@@ -2004,7 +2022,7 @@ class GitRepository(object):
         try:
             if not os.path.exists(abspath):
                 os.makedirs(abspath)
-            stderr = ''
+            stderr = b''
             try:
                 for out in klass.__git_inout(command='init',
                                              args=args.args,
@@ -2015,7 +2033,7 @@ class GitRepository(object):
                                              capture_stdout=True):
                     stderr += out[1]
             except GitRepositoryError:
-                raise GitRepositoryError("Error running git init: %s" % stderr)
+                raise GitRepositoryError("Error running git init: %s" % stderr.decode().strip())
             except Exception as excobj:
                 raise GitRepositoryError("Error running git init: %s" % excobj)
             if description:
@@ -2025,7 +2043,7 @@ class GitRepository(object):
             return klass(abspath)
         except OSError as err:
             raise GitRepositoryError("Cannot create Git repository at '%s': %s"
-                                     % (abspath, err[1]))
+                                     % (abspath, err))
         return None
 
     @classmethod
@@ -2068,7 +2086,7 @@ class GitRepository(object):
         try:
             if not os.path.exists(abspath):
                 os.makedirs(abspath)
-            stderr = ''
+            stderr = b''
             try:
                 for out in klass.__git_inout(command='clone',
                                              args=args.args,
@@ -2079,7 +2097,7 @@ class GitRepository(object):
                                              capture_stdout=True):
                     stderr += out[1]
             except GitRepositoryError:
-                raise GitRepositoryError("Error running git clone: %s" % stderr)
+                raise GitRepositoryError("Error running git clone: %s" % stderr.decode())
             except Exception as excobj:
                 raise GitRepositoryError("Error running git clone: %s" % excobj)
 
