@@ -17,7 +17,7 @@
 #    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """provides some rpm source package related helpers"""
 
-import commands
+import subprocess
 import sys
 import os
 import re
@@ -38,6 +38,11 @@ from gbp.rpm.linkedlist import LinkedList
 from gbp.rpm.lib_rpm import librpm, get_librpm_log
 
 
+def _decode(s):
+    if s is not None:
+        return s.decode()
+
+
 class NoSpecError(Exception):
     """Spec file parsing error"""
     pass
@@ -72,8 +77,8 @@ class SrcRpmFile(object):
     @property
     def version(self):
         """Get the (downstream) version of the RPM package"""
-        version = dict(upstreamversion = self.rpmhdr[librpm.RPMTAG_VERSION],
-                       release = self.rpmhdr[librpm.RPMTAG_RELEASE])
+        version = dict(upstreamversion=self.rpmhdr[librpm.RPMTAG_VERSION].decode(),
+                       release=self.rpmhdr[librpm.RPMTAG_RELEASE].decode())
         if self.rpmhdr[librpm.RPMTAG_EPOCH] is not None:
             version['epoch'] = str(self.rpmhdr[librpm.RPMTAG_EPOCH])
         return version
@@ -81,17 +86,17 @@ class SrcRpmFile(object):
     @property
     def name(self):
         """Get the name of the RPM package"""
-        return self.rpmhdr[librpm.RPMTAG_NAME]
+        return self.rpmhdr[librpm.RPMTAG_NAME].decode()
 
     @property
     def upstreamversion(self):
         """Get the upstream version of the RPM package"""
-        return self.rpmhdr[librpm.RPMTAG_VERSION]
+        return self.rpmhdr[librpm.RPMTAG_VERSION].decode()
 
     @property
     def packager(self):
         """Get the packager of the RPM package"""
-        return self.rpmhdr[librpm.RPMTAG_PACKAGER]
+        return _decode(self.rpmhdr[librpm.RPMTAG_PACKAGER])
 
     def unpack(self, dest_dir):
         """
@@ -133,7 +138,7 @@ class SpecFile(object):
             self.specfile = os.path.basename(filename)
             self.specdir = os.path.dirname(os.path.abspath(filename))
             try:
-                with open(filename) as spec_file:
+                with open(filename, errors='ignore') as spec_file:
                     for line in spec_file.readlines():
                         self._content.append(line)
             except IOError as err:
@@ -159,13 +164,13 @@ class SpecFile(object):
 
         # Other initializations
         source_header = self._specinfo.packages[0].header
-        self.name = source_header[librpm.RPMTAG_NAME]
-        self.upstreamversion = source_header[librpm.RPMTAG_VERSION]
-        self.release = source_header[librpm.RPMTAG_RELEASE]
+        self.name = source_header[librpm.RPMTAG_NAME].decode()
+        self.upstreamversion = source_header[librpm.RPMTAG_VERSION].decode()
+        self.release = source_header[librpm.RPMTAG_RELEASE].decode()
         # rpm-python returns epoch as 'long', convert that to string
         self.epoch = str(source_header[librpm.RPMTAG_EPOCH]) \
-            if source_header[librpm.RPMTAG_EPOCH] != None else None
-        self.packager = source_header[librpm.RPMTAG_PACKAGER]
+            if source_header[librpm.RPMTAG_EPOCH] is not None else None
+        self.packager = _decode(source_header[librpm.RPMTAG_PACKAGER])
         self._tags = {}
         self._special_directives = defaultdict(list)
         self._gbp_tags = defaultdict(list)
@@ -184,7 +189,7 @@ class SpecFile(object):
     def _parse_filtered_spec(self, skip_tags):
         """Parse a filtered spec file in rpm-python"""
         skip_tags = [tag.lower() for tag in skip_tags]
-        with tempfile.NamedTemporaryFile(prefix='gbp') as filtered:
+        with tempfile.NamedTemporaryFile(prefix='gbp', mode='w+') as filtered:
             filtered.writelines(str(line) for line in self._content
                     if str(line).split(":")[0].strip().lower() not in skip_tags)
             filtered.flush()
@@ -194,7 +199,7 @@ class SpecFile(object):
                 librpm.spec(filtered.name)
                 return librpm.spec(filtered.name)
             except ValueError as err:
-                rpmlog = get_librpm_log()
+                rpmlog = [log.decode() for log in get_librpm_log()]
                 gbp.log.debug("librpm log:\n        %s" %
                                 "\n        ".join(rpmlog))
                 raise GbpError("RPM error while parsing %s: %s (%s)" %
@@ -243,7 +248,7 @@ class SpecFile(object):
     def sources(self):
         """Get all source tags as a dict"""
         sources = {}
-        for src in self._sources().values():
+        for src in list(self._sources().values()):
             sources[src['num']] = src['linevalue']
         return sources
 
@@ -303,7 +308,7 @@ class SpecFile(object):
             tagvalue = None
         # We don't support "multivalue" tags like "Provides:" or "SourceX:"
         # Rpm python doesn't support many of these, thus the explicit list
-        if type(tagvalue) is int or type(tagvalue) is long:
+        if isinstance(tagvalue, int):
             tagvalue = str(tagvalue)
         elif type(tagvalue) is list or tagname in self._listtags:
             tagvalue = None
@@ -613,7 +618,7 @@ class SpecFile(object):
         macro_prev = None
         ignored = self.ignorepatches
         # Remove 'Patch:̈́' tags
-        for tag in self._patches().values():
+        for tag in list(self._patches().values()):
             if not tag['num'] in ignored:
                 tag_prev = self._delete_tag('patch', tag['num'])
                 # Remove a preceding comment if it seems to originate from GBP
@@ -694,7 +699,7 @@ class SpecFile(object):
             macro_line = self._content.insert_after(macro_line, comment_text)
             macro_line = self._set_special_macro('patch', patchnum, '-p1',
                                                  macro_line)
-            for cmd, args in cmds.iteritems():
+            for cmd, args in cmds.items():
                 if cmd in ('if', 'ifarch'):
                     self._content.insert_before(macro_line, '%%%s %s\n' %
                                                 (cmd, args))
@@ -779,7 +784,7 @@ class SpecFile(object):
         """
         orig = None
         sources = self.sources()
-        for num, filename in sorted(sources.iteritems()):
+        for num, filename in sorted(sources.items()):
             src = {'num': num, 'filename': os.path.basename(filename),
                    'uri': filename}
             src['filename_base'], src['archive_fmt'], src['compression'] = \
@@ -803,9 +808,9 @@ def parse_srpm(srpmfile):
     """parse srpm by creating a SrcRpmFile object"""
     try:
         srcrpm = SrcRpmFile(srpmfile)
-    except IOError, err:
+    except IOError as err:
         raise GbpError("Error reading src.rpm file: %s" % err)
-    except librpm.error, err:
+    except librpm.error as err:
         raise GbpError("RPM error while reading src.rpm: %s" % err)
 
     return srcrpm
@@ -853,11 +858,11 @@ def guess_spec_repo(repo, treeish, topdir='', recursive=True, preferred_name=Non
     """
     topdir = topdir.rstrip('/') + ('/') if topdir else ''
     try:
-        file_list = [nam for (mod, typ, sha, nam) in
-                    repo.list_tree(treeish, recursive, topdir) if typ == 'blob']
+        file_list = [nam.decode() for (mod, typ, sha, nam) in
+                     repo.list_tree(treeish, recursive, topdir) if typ == 'blob']
     except GitRepositoryError as err:
         raise NoSpecError("Cannot find spec file from treeish %s, Git error: %s"
-                            % (treeish, err))
+                          % (treeish, err))
     spec_path = guess_spec_fn(file_list, preferred_name)
     return spec_from_repo(repo, treeish, spec_path)
 
@@ -865,7 +870,7 @@ def guess_spec_repo(repo, treeish, topdir='', recursive=True, preferred_name=Non
 def spec_from_repo(repo, treeish, spec_path):
     """Get and parse a spec file from a give Git treeish"""
     try:
-        spec = SpecFile(filedata=repo.show('%s:%s' % (treeish, spec_path)))
+        spec = SpecFile(filedata=repo.show('%s:%s' % (treeish, spec_path)).decode(errors='ignore'))
         spec.specdir = os.path.dirname(spec_path)
         spec.specfile = os.path.basename(spec_path)
         return spec
